package svc

import (
	"applet/app/db"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"fmt"
	"strings"
	"xorm.io/xorm"
)

func JpushRecord(eg *xorm.Engine) {
	PessimismLockValue := "JpushRecord"
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		return
	}
	defer ch.Release()
	key := fmt.Sprintf("JpushRecord")
	//TODO::增加“悲观锁”防止串行
	getString, _ := cache.GetString(key)
	if getString == PessimismLockValue {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次未执行完")
		return
	}
	cache.SetEx(key, PessimismLockValue, 3600*8) //8小时

	var data []model.JpushRecord
	eg.Where("is_run=?", 0).Limit(3).Asc("id").Find(&data)
	for _, v := range data {
		p := 1
		if v.Target == "0" { //全量用户
			arg := md.JpushRecordFundData{Id: utils.IntToStr(v.Id), Target: v.Target, Platform: v.Platform, Title: v.Title, Content: v.Content}
			err := ch.PublishV2(md.EggJpushRecordQueueExchange, arg, md.EggJpushRecordQueue)
			if err != nil {
				ch.PublishV2(md.EggJpushRecordQueueExchange, arg, md.EggJpushRecordQueue)
			}
		}
		if v.Target == "1" { //部分用户
			MoreJpushRecordSend(eg, ch, v)
		}
		if v.Target == "2" { //等级
			commJpushRecordSend(eg, ch, v, p, v.Level)
		}
		if v.Target == "3" { //标签
			tagJpushRecordSend(eg, ch, v, p)
		}
		v.IsRun = 1
		eg.Where("id=?", v.Id).Cols("is_run").Update(&v)
	}
	cache.Del(key)
}

func commJpushRecordSend(eg *xorm.Engine, ch *rabbit.Channel, data model.JpushRecord, p int, level string) {
	var userList []model.User
	sess := eg.Where("1=1")
	if level != "" {
		var tmp []string
		json.Unmarshal([]byte(level), &tmp)
		sess.In("level", tmp)
	}
	sess.Asc("id").Limit(1000, p).Find(&userList)
	uids := ""
	if len(userList) == 0 {
		return
	}
	for _, v := range userList {
		if v.Phone == "" {
			continue
		}
		if uids == "" {
			uids += utils.Int64ToStr(v.Id)
		} else {
			uids += "," + utils.Int64ToStr(v.Id)
		}
	}
	if uids == "" {
		return
	}
	arg := md.JpushRecordFundData{UserId: uids, Target: data.Target, Platform: data.Platform, Title: data.Title, Content: data.Content}
	if len(userList) < 1000 {
		arg.Id = utils.IntToStr(data.Id)
	}
	err := ch.PublishV2(md.EggJpushRecordQueueExchange, arg, md.EggJpushRecordQueue)
	if err != nil {
		ch.PublishV2(md.EggJpushRecordQueueExchange, arg, md.EggJpushRecordQueue)
	}
	if len(userList) == 1000 {
		p++
		commJpushRecordSend(eg, ch, data, p, level)
		return
	}
	return

}
func tagJpushRecordSend(eg *xorm.Engine, ch *rabbit.Channel, data model.JpushRecord, p int) {
	sql := `
	SELECT * FROM user_tag_records utr
left join user u on u.id=utr.uid
WHERE %s GROUP BY u.id ORDER BY u.id asc %s
`
	tagList := make([]string, 0)
	json.Unmarshal([]byte(data.TagId), &tagList)
	if len(tagList) == 0 {
		return
	}
	where := "tag_id in(" + strings.Join(tagList, ",") + ")"
	limit := " limit " + utils.IntToStr((p-1)*1000) + ",1000"
	sql = fmt.Sprintf(sql, where, limit)
	userList, _ := db.QueryNativeString(eg, sql)
	uids := ""
	if len(userList) == 0 {
		return
	}
	for _, v := range userList {
		if v["phone"] == "" {
			continue
		}
		if uids == "" {
			uids += v["id"]
		} else {
			uids += "," + v["id"]
		}
	}
	if uids == "" {
		return
	}
	arg := md.JpushRecordFundData{UserId: uids, Target: data.Target, Platform: data.Platform, Title: data.Title, Content: data.Content}
	if len(userList) < 1000 {
		arg.Id = utils.IntToStr(data.Id)
	}
	err := ch.PublishV2(md.EggJpushRecordQueueExchange, arg, md.EggJpushRecordQueue)
	if err != nil {
		ch.PublishV2(md.EggJpushRecordQueueExchange, arg, md.EggJpushRecordQueue)
	}
	if len(userList) == 1000 {
		p++
		tagJpushRecordSend(eg, ch, data, p)
		return
	}
	return

}
func MoreJpushRecordSend(eg *xorm.Engine, ch *rabbit.Channel, data model.JpushRecord) {
	userList := make([]string, 0)
	json.Unmarshal([]byte(data.UserId), &userList)
	tmpUser := make([]string, 0)
	for k, v := range userList {
		if len(tmpUser) == 1000 || (k+1 >= len(userList) && len(tmpUser) < 1000) {
			arg := md.JpushRecordFundData{UserId: strings.Join(tmpUser, ","), Target: data.Target, Platform: data.Platform, Title: data.Title, Content: data.Content}
			if k+1 >= len(userList) {
				arg.Id = utils.IntToStr(data.Id)
			}
			err := ch.PublishV2(md.EggJpushRecordQueueExchange, arg, md.EggJpushRecordQueue)
			if err != nil {
				ch.PublishV2(md.EggJpushRecordQueueExchange, arg, md.EggJpushRecordQueue)
			}
			tmpUser = []string{}
		}
		tmpUser = append(tmpUser, v)
	}
	return
}
